diff -u oggenc-orig\encode.h oggenc\encode.h
--- oggenc-orig\encode.h	Wed Jul 31 20:03:41 2002
+++ oggenc\encode.h	Fri Aug 16 21:39:27 2002
@@ -18,7 +18,7 @@
 void *timer_start(void);
 double timer_time(void *);
 void timer_clear(void *);
-int create_directories(char *);
+int create_directories(char *, int);
 
 void update_statistics_full(char *fn, long total, long done, double time);
 void update_statistics_notime(char *fn, long total, long done, double time);
@@ -82,6 +82,8 @@
 
     int resamplefreq;
     int downmix;
+
+    int isutf8;
 
 	unsigned int serial;
 } oe_options;
diff -u oggenc-orig\oggenc.c oggenc\oggenc.c
--- oggenc-orig\oggenc.c	Mon Aug 12 18:30:26 2002
+++ oggenc\oggenc.c	Fri Aug 16 21:39:16 2002
@@ -24,7 +24,6 @@
 #include "utf8.h"
 #include "i18n.h"
 
-
 #define VERSION_STRING "OggEnc v1.0 (libvorbis 1.0)\n"
 #define COPYRIGHT "(c) 2000-2002 Michael Smith <msmith@labyrinth.net.au>\n"
 
@@ -59,6 +58,9 @@
     {"resample",1,0,0},
     {"downmix", 0,0,0},
     {"advanced-encode-option", 1, 0, 0},
+#ifndef _WIN32
+    {"utf8", 0,0,0},
+#endif
 	{NULL,0,0,0}
 };
 	
@@ -77,7 +79,7 @@
 	oe_options opt = {NULL, 0, NULL, 0, NULL, 0, NULL, 0, NULL, 
 			  0, NULL, 0, NULL, 0, NULL, 0, 0, 0,16,44100,2, 0, NULL,
 			  DEFAULT_NAMEFMT_REMOVE, DEFAULT_NAMEFMT_REPLACE, 
-			  NULL, 0, -1,-1,-1,.3,-1,0, 0,0}; 
+			  NULL, 0, -1,-1,-1,.3,-1,0, 0, 0, 0}; 
 
 	int i;
 
@@ -85,6 +87,8 @@
 	int numfiles;
 	int errors=0;
 
+    get_args_from_ucs16(&argc, &argv, &opt.isutf8);
+
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
@@ -165,7 +169,7 @@
 		}
 		else
 		{
-			in = fopen(infiles[i], "rb");
+            in = oggenc_fopen(infiles[i], "rb", opt.isutf8);
 
 			if(in == NULL)
 			{
@@ -261,7 +265,7 @@
             }
 
             /* Create any missing subdirectories, if possible */
-            if(create_directories(out_fn)) {
+            if(create_directories(out_fn, opt.isutf8)) {
                 if(closein)
                     fclose(in);
 				fprintf(stderr, _("ERROR: Could not create required subdirectories for output filename \"%s\"\n"), out_fn);
@@ -270,7 +274,7 @@
 				continue;
             }
 
-			out = fopen(out_fn, "wb");
+            out = oggenc_fopen(out_fn, "wb", opt.isutf8);
 			if(out == NULL)
 			{
 				if(closein)
@@ -286,8 +290,14 @@
 		/* Now, set the rest of the options */
 		enc_opts.out = out;
 		enc_opts.comments = &vc;
-		enc_opts.filename = out_fn;
-		enc_opts.infilename = infiles[i];
+        if (opt.isutf8) {
+            /* TODO: error checking here */
+            utf8_decode(out_fn, &enc_opts.filename);
+            utf8_decode(infiles[i], &enc_opts.infilename);
+        } else {
+            enc_opts.filename = out_fn;
+            enc_opts.infilename = infiles[i];
+        }
         enc_opts.managed = opt.managed;
 		enc_opts.bitrate = opt.nominal_bitrate; 
 		enc_opts.min_bitrate = opt.min_bitrate;
@@ -345,6 +355,10 @@
 
 		if(out_fn) free(out_fn);
         if(opt.outfile) free(opt.outfile);
+        if (opt.isutf8) {
+            free(enc_opts.filename);
+            free(enc_opts.infilename);
+        }
 		vorbis_comment_clear(&vc);
 		if(!opt.rawmode) 
 			format->close_func(enc_opts.readdata);
@@ -439,6 +453,11 @@
 		" In this mode, output is to stdout unless an output filename is specified\n"
 		" with -o\n"
 		"\n"), VERSION_STRING, COPYRIGHT);
+
+/* If we decide to document this, we must not document it on WIN32.
+        " --utf8               Tells oggenc that the command line parameters date, title,\n"
+		"                      album, artist, genre, and comment are already in UTF8\n"
+*/
 }
 
 static int strncpy_filtered(char *dst, char *src, int len, char *remove_list, 
@@ -590,6 +609,9 @@
                 else if(!strcmp(long_options[option_index].name, "downmix")) {
                     opt->downmix = 1;
                 }
+                else if(!strcmp(long_options[option_index].name, "utf8")) {
+                    opt->isutf8 = 1;
+                }
                 else if(!strcmp(long_options[option_index].name, "advanced-encode-option")) {
                     char *arg = strdup(optarg);
                     char *val;
@@ -799,7 +821,15 @@
 static void add_tag(vorbis_comment *vc, oe_options *opt,char *name, char *value)
 {
 	char *utf8;
-	if(utf8_encode(value, &utf8) >= 0)
+    if (opt->isutf8) {
+        /* TODO: Add checking here to verify that 'value' really is
+                 a valid utf8 string */
+		if(name == NULL)
+			vorbis_comment_add(vc, value);
+		else
+			vorbis_comment_add_tag(vc, name, value);
+    }
+    else if(utf8_encode(value, &utf8) >= 0)
 	{
 		if(name == NULL)
 			vorbis_comment_add(vc, utf8);
diff -u oggenc-orig\platform.c oggenc\platform.c
--- oggenc-orig\platform.c	Thu Jul 11 12:48:15 2002
+++ oggenc\platform.c	Fri Aug 16 21:39:07 2002
@@ -23,6 +23,10 @@
 #include <time.h>
 #endif
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
+
 #if defined(_WIN32) && defined(_MSC_VER)
 
 void setbinmode(FILE *f)
@@ -124,7 +128,7 @@
 
 #endif
 
-int create_directories(char *fn)
+int create_directories(char *fn, int isutf8)
 {
     char *end, *start;
     struct stat statbuf;
@@ -138,10 +142,19 @@
 
     while((end = strpbrk(start+1, PATH_SEPS)) != NULL)
     {
+        int rv;
         memcpy(segment, fn, end-fn);
         segment[end-fn] = 0;
 
-        if(stat(segment,&statbuf)) {
+#ifdef _WIN32
+        if (isutf8) {
+            wchar_t seg[MAX_PATH+1];
+            MultiByteToWideChar(CP_UTF8, 0, segment, -1, seg, MAX_PATH+1);
+            rv = _wstat(seg,&statbuf);
+        } else
+#endif
+            rv = stat(segment,&statbuf);
+        if(rv) {
             if(errno == ENOENT) {
                 if(mkdir(segment, 0777)) {
                     fprintf(stderr, _("Couldn't create directory \"%s\": %s\n"),
@@ -178,5 +191,85 @@
 
 }
 
+#ifdef _WIN32
 
+FILE *oggenc_fopen(char *fn, char *mode, int isutf8)
+{
+    if (isutf8) {
+        wchar_t wfn[MAX_PATH+1];
+        wchar_t wmode[32];
+        MultiByteToWideChar(CP_UTF8, 0, fn, -1, wfn, MAX_PATH+1);
+        MultiByteToWideChar(CP_ACP, 0, mode, -1, wmode, 32);
+        return _wfopen(wfn, wmode);
+    } else
+        return fopen(fn, mode);
+}
+
+typedef WINSHELLAPI LPWSTR *  (APIENTRY *tCommandLineToArgvW)(LPCWSTR lpCmdLine, int*pNumArgs);
+
+void get_args_from_ucs16(int *argc, char ***argv, int *isutf8)
+{
+    OSVERSIONINFO vi;
+    vi.dwOSVersionInfoSize = sizeof(vi);
+    GetVersionEx(&vi);
+
+    /* We only do NT4 and more recent.*/
+    /* It would be relatively easy to add NT3.5 support. Is anyone still using NT3? */
+    /* It would be relatively hard to add 9x support. Fortunately, 9x is
+       a lost cause for unicode support anyway. */
+    if (vi.dwPlatformId == VER_PLATFORM_WIN32_NT && vi.dwMajorVersion >= 4) {
+        int newargc;
+        int sizeofargs = 0;
+        int a, count;
+        char *argptr;
+        char **newargv = NULL;
+        LPWSTR *ucs16argv = NULL;
+        tCommandLineToArgvW pCommandLineToArgvW = NULL;
+        HMODULE hLib = NULL;
+        
+        hLib = LoadLibrary("shell32.dll");
+        if (!hLib)
+            goto bail;
+        pCommandLineToArgvW = (tCommandLineToArgvW)GetProcAddress(hLib, "CommandLineToArgvW");
+        if (!pCommandLineToArgvW)
+            goto bail;
+
+        ucs16argv = pCommandLineToArgvW(GetCommandLineW(), &newargc);
+        if (!ucs16argv)
+            goto bail;
+
+        for (a=0; a<newargc; a++) {
+            count = WideCharToMultiByte(CP_UTF8, 0, ucs16argv[a], -1,
+                NULL, 0, NULL, NULL);
+            if (count == 0)
+                goto bail;
+            sizeofargs += count;
+        }
 
+        newargv = malloc((newargc * sizeof(char *)) + sizeofargs);
+        argptr = (char *)(&newargv[newargc]);
+
+        for (a=0; a<newargc; a++) {
+            count = WideCharToMultiByte(CP_UTF8, 0, ucs16argv[a], -1,
+                argptr, sizeofargs, NULL, NULL);
+            if (count == 0)
+                goto bail;
+
+            newargv[a] = argptr;
+            argptr += count;
+            sizeofargs -= count;
+        }
+
+        *argc = newargc;
+        *argv = newargv;
+        *isutf8 = 1;
+
+bail:
+        if (hLib != NULL)
+            FreeLibrary(hLib);
+        if (ucs16argv != NULL)
+            GlobalFree(ucs16argv);
+    }
+}
+
+#endif
diff -u oggenc-orig\platform.h oggenc\platform.h
--- oggenc-orig\platform.h	Tue Apr 16 14:18:02 2002
+++ oggenc\platform.h	Fri Aug 16 21:38:48 2002
@@ -29,5 +29,19 @@
 
 #endif
 
+#ifdef _WIN32
+
+extern FILE *oggenc_fopen(char *fn, char *mode, int isutf8);
+extern void get_args_from_ucs16(int *argc, char ***argv, int *isutf8);
+
+#else
+
+/* presume that people aren't going to put utf8 filenames on the command line. */
+/* Is this a valid presumption? */
+#define oggenc_fopen(x,y,z) fopen(x,y)
+#define get_args_from_ucs16(x,y,z) { }
+
+#endif
+
 #endif /* __PLATFORM_H */
 

𸬵܆+޲mb+rz΂
k7&zzmb+ {.n++%lzwZ,j+n+zު笷rj)
'+a{
+v칻&ފ{ay,rجy'rzgǧ-b"Vފ	~)mz